/******************************************************
 * Copyright eNoe 2011 - All rights reserved *
 * Author: Séverin MARCOMBES                          *
 ******************************************************/
		 /**
		 * CategoryMenuServer
		 * @constructor
		 * @param {string} fetchUrlCategories
		 * @param {string} fetchUrlPrograms
		 * @param {int} updatePeriod
		 */
		function CategoryMenuServer(fetchUrlCategories, fetchUrlPrograms, updatePeriod)
		{
			var _that						= this;

			_that.fetchUrlCategories		= fetchUrlCategories;
			_that.fetchUrlPrograms			= fetchUrlPrograms;
			_that.updatePeriod				= updatePeriod;
			_that._data						= null;
			_that.timer              		= null;
			_that._userId					= 0;
			
//--------------------------------------------------------------
//--------------------------------------------------------------
			_that._getUrl					= function(url)
			{
				var reg=new RegExp("(%id%)", "g");
          
         		return url.replace(reg,"" + _that._userId);
			};
//--------------------------------------------------------------
			_that.getData					= function()
			{
				return _that._data;
			};
//--------------------------------------------------------------
			
			_that._buidDataNode				= function(categoryId, categoriesIndex, ProgramsIndex)
			{
				var data		= new Object();
				
				data['icon']		= categoriesIndex[categoryId]['icon'];
				data['title']		= categoriesIndex[categoryId]['name'];
				
				if(categoriesIndex[categoryId]['style'])
				{
					data['style']	= categoriesIndex[categoryId]['style'];
				}
				else
				{
					data['style']	= new Object();
				}
				
				//Put subcategeries in blue
				if(categoryId == "root")
				{
					for(var i=0; i<categoriesIndex[categoryId]['category_ids'].length; i++)
					{
					  if(!categoriesIndex[categoriesIndex[categoryId]['category_ids'][i]]['style'])
					  {
					    categoriesIndex[categoriesIndex[categoryId]['category_ids'][i]]['style']		= new Object();
					  }
					  categoriesIndex[categoriesIndex[categoryId]['category_ids'][i]]['style']['color'] = "black";
					}
				}

				if((categoriesIndex[categoryId]['program_ids']) && (!data['style']['color']))
				{
					data['style']['color'] = "rgb(0,91,168)";
				}
				
				
				data['children']	= new Array();
				
				for(var i=0; i<categoriesIndex[categoryId]['category_ids'].length; i++)
				{
					data['children'].push(_that._buidDataNode(categoriesIndex[categoryId]['category_ids'][i], categoriesIndex, ProgramsIndex));
				}
				
				var defaultIcon = navigatorAbstractionLayer.getMenuNodeDefaultIcon(); 
				
				if(categoriesIndex[categoryId]['program_ids'])
				{
					var children        = new Object();
					var childrenTitles  = new Array();
          
					for(var i=0; i<categoriesIndex[categoryId]['program_ids'].length; i++)
					{
						var program 		= new Object();
					
						program['title'] 	= ProgramsIndex[categoriesIndex[categoryId]['program_ids'][i]]['title'];
						program['icon']		= ProgramsIndex[categoriesIndex[categoryId]['program_ids'][i]]['icon'];
						
						if(program['icon'] == "")
						{
							program['icon']	= defaultIcon;
						}
						
						program['url']		= _that._getUrl(ProgramsIndex[categoriesIndex[categoryId]['program_ids'][i]]['url']); 
						
						//Manage domain pre-activation
						program['domainId']	= ProgramsIndex[categoriesIndex[categoryId]['program_ids'][i]]['domainId'];
				
						children[program['title'].toUpperCase()]    = program;
						childrenTitles.push(program['title'].toUpperCase());
					}
					
					childrenTitles.sort();
					
					for(var i=0; i<childrenTitles.length; i++)
					{
            			data['children'].push(children[childrenTitles[i]]);
					}
				}
				
				return data;
			};
//--------------------------------------------------------------			
			_that._generateInternalData		= function(categoriesIndex, ProgramsIndex, SitesIndex)
			{
				//For pre-activation: add a domainId entry to each program entry
				for(var site in SitesIndex)
				{
					for(var j=0; j<SitesIndex[site]['urls'].length; j++)
					{
						ProgramsIndex[SitesIndex[site]['urls'][j]]['domainId']	= site; 
					}
				}
				
				//Add program_ids to the category objects listed in the categoriesIndex	
				for(var program in ProgramsIndex)
				{
					for(var i=0; i<ProgramsIndex[program]['category_ids'].length; i++)
					{
						if(!categoriesIndex[ProgramsIndex[program]['category_ids'][i]]['program_ids'])
						{
							categoriesIndex[ProgramsIndex[program]['category_ids'][i]]['program_ids'] = new Array();
						}
						
						categoriesIndex[ProgramsIndex[program]['category_ids'][i]]['program_ids'].push(program);
					}
				}
				
				//Add style customizations
				categoriesIndex["selection"]['style'] = 
				{
					'fontWeight':		"bold"
				};
				
				//Build a clean tree structure from that data
				_that._data = _that._buidDataNode("root", categoriesIndex, ProgramsIndex);
				
				//Save data for next unconnected uses
				localStorage["config_manager_data"] = JSON.stringify(_that._data);		
			};
//--------------------------------------------------------------	
			//Refresh the content of the menu with updated data
			_that.update						= function()
			{
				//1. Get the category hierarchy
				wget
				(
					_that.fetchUrlCategories,
					
					function(response, args)
					{
						
						if((response) && (typeof(response) == "object"))
						{
							//2. Get the programs
							wget
							(
								_that.fetchUrlPrograms,
								
								function(response2, args2)
								{
									
									if(response2 && response2['programs'])
									{
										//3. Reinitialize internal data with the content fetched
										_that._generateInternalData(args2['categories'], response2['programs'], response2['sites']);
									
										//4. Refresh the DOM object
										//_that._processData();
									}
								},
								
								{
									'_this':		_that,
									'categories':	response
								}
							);
						}
					},
					
					{
						'_this':	_that
					}
				);
			};
//--------------------------------------------------------------
			_that.setUserId					= function(userId)
			{
				_that._userId 				= userId;
				_that.update();
			};
//--------------------------------------------------------------
			_that.setUpdatePeriod    	= function(updatePeriod)
			{
				_that.updatePeriod 		= updatePeriod;

				clearInterval(_that.timer);

				_that.timer				= setInterval
				(
				  function()
				  {
				    _that.update();
				  },
				  
				  (_that.updatePeriod * 1000)
				);	

			};
//--------------------------------------------------------------
//--------------------------------------------------------------
			
			//Generate the DOM object
			if(localStorage["config_manager_data"])
			{
				try
				{
					_that._data        = JSON.parse(localStorage["config_manager_data"]);
				}
				catch(exc)
				{
					_that.update();
				}
			}
			else
			{
				_that.update();
			}
			
			//Set a timer to refresh the menu periodically
			 _that.setUpdatePeriod(_that.updatePeriod);
		}
