/******************************************************
 * Copyright eNoe 2011 - All rights reserved *
 * Author: Séverin MARCOMBES                          *
 ******************************************************/
/**
 * Database
 * @constructor
 * @param {string} fetchUrl
 * @param {int} ttl
 */
function Database(fetchUrl,fetchUrl2,ttl) 
{
  var _that               = this;

  _that.fetchUrl           = fetchUrl;
  _that.fetchUrl2           = fetchUrl2;  
  _that.ttl                = ttl;

  _that['_d']              =
  {
    "domains":              {},
    "sites":                {},
    "programs":             {} 
    //,
//    domainMap:            {}
  };
  
  _that['_d2']              =
  {
    "referrers_list":              {},
    "default_messages":                {}
  };  

  _that._lastFetchDate     = 0;
  _that._request           = null;

//--------------------------------------------------------------
//--------------------------------------------------------------
  _that._storeData         = function(data)
  {
    if
    (
        (data)
      &&(data['domains'])
      &&(data['sites'])
      &&(data['programs'])
      &&(typeof(data)=="object")
      &&(typeof(data['domains'])=="object")
      &&(typeof(data['sites'])=="object")
      &&(typeof(data['programs'])=="object")
    )
    { 
      //Adding domainId to each program for easier manipulation
      for(var program in data['programs'])
      {
      	for(var site in data['sites'])
      	{
      		for(var i=0; i<data['sites'][site]['urls'].length; i++)
      		{
      			if(data['sites'][site]['urls'][i]==program)
      			{
      				data['programs'][program]['domainId'] = site;
      			}
      		}
      	}
      }
    
	  _that['_d']            = data; 
    
      _that._lastFetchDate   = Math.round(new Date().getTime() / 1000); 
    }
  };
//--------------------------------------------------------------
  _that.fetch              = function(callback, args)
  {
    try
    {
      _that._request.abort();
    }
    catch(exc)
    {}

    _that._request = new XMLHttpRequest();

    _that._request.open('GET', _that.fetchUrl, true); //TODO: Add a parameter behind the URL to avoid cache effects

    _that._request.onreadystatechange = function()
    {
      if (_that._request.readyState == 4)
      {
        if (_that._request.status == 200)
        {
          try
          {
            var data = JSON.parse(_that._request.responseText);
          
            _that._storeData(data);

            //callback(args);
          }
          catch(err)
          {
            //Do nothing
          } 
        }
        else
        {}
      }
    };

    _that._request.send();
    
    
    _that._request2 = new XMLHttpRequest();

    _that._request2.open('GET', _that.fetchUrl2, true); //TODO: Add a parameter behind the URL to avoid cache effects

    _that._request2.onreadystatechange = function()
    {
      if (_that._request2.readyState == 4)
      {
        if (_that._request2.status == 200)
        {
          try
          {
            var data2 = JSON.parse(_that._request2.responseText);

	  _that['_d2']            = data2; 

            callback(args);
          }
          catch(err)
          {
            //Do nothing
          } 
        }
        else
        {}
      }
    };

    _that._request2.send();    
  };
//--------------------------------------------------------------
  _that.update           = function(callback, args)
  {
    var currentTime = Math.round(new Date().getTime() / 1000);

    if(currentTime > (_that._lastFetchDate + _that.ttl))
    {
      _that.fetch(callback, args);
    }
    else
    {
      callback(args);
    }
  };
//--------------------------------------------------------------
  _that.getDomainInfo               = function(domain)
  {
    var info = new Object();

    var _domain = _that['_d']['domains'][domain];
    if(_domain)
    {
      var _site = _that['_d']['sites'][_domain];
      if(_site)
      {
        info['domain']     = domain;
        info['title']      = _site['title'];
        info['text']      = _site['text'];        
        info['codes']      = _site['codes'];
        info['specials']   = _site['specials'];
        info['urls']       = new Array();
        
        for(var i=0; i < _site['urls'].length; i++)
        {
          if(_that['_d']['programs'][_site['urls'][i]])
          {
            info['urls'].push(_that['_d']['programs'][_site['urls'][i]]['url']);
          }
        }
      }
    }
    
    return info;
  }
//--------------------------------------------------------------
//--------------------------------------------------------------
  _that.getDomainInfoById          = function(domainId, callback, args)
  {
    _that.update
    (
      function(args)
      {    
        var _site = _that['_d']['sites'][args['domainId']];
          
        if(_site)
        {
          var info = new Object();
        
          info['title']      = _site['title'];
          info['codes']      = _site['codes'];
          info['text']      = _site['text'];             
          info['specials']   = _site['specials'];
          info['urls']       = new Array();
            
          for(var i=0; i < _site['urls'].length; i++)
          {
            if(_that['_d']['programs'][_site['urls'][i]])
            {
              info['urls'].push(_that['_d']['programs'][_site['urls'][i]]['url']);
            }
          }
          
          //Return info (only if not empty)
          args['callback'](info, args['callbackArgs']);
        }
      },
      
      {
        '_this':           _that,
        'domainId':        domainId,
        'callback':        callback,
        'callbackArgs':    args
      }
    );
  }
  
//--------------------------------------------------------------    
  _that.ifDomainIsConcerned        = function(domain, callback, callbackArgs)
  {
    _that.update
    (
      function(args)
      {        
        //We don't get the full domain names from the server
        //So.. Let's do regular expressions
          var domainsFound = new Array();
        
          for(var domain in _that['_d']['domains'])
          {
              //We put in a table all domain names which could correspond to our target domain
              var exp = new RegExp(domain + "$", "g");
              
              if(exp.test(args['domain']))
              {
                domainsFound.push(domain);
              }
          }
        
        //If we found corresponding domain names, we continue
        if(domainsFound.length > 0)
        {
          // One our more domains were found.
          // We prepare to callback.
          var domainFound = "";

          // If several domains correspond to our host, we focus on the longest one  
          if(domainsFound.length > 1)
          {
            for(var i=0; i<domainsFound.length; i++)
            {
              if(domainsFound[i].length > domainFound.length)
              {
                domainFound = domainsFound[i];
              }
            }
          }
          else
          {
            domainFound = domainsFound[0];
          }
          
          // Let's return the information attached to the domain
          var domainId = _that['_d']['domains'][domainFound];
          args['callback'](domainId, args['callbackArgs']);
        }
      },
    
      {
        '_this':          _that,
        'domain':         domain,
        'callback':       callback,
        'callbackArgs':   callbackArgs
      }
    )
  };
//--------------------------------------------------------------
//--------------------------------------------------------------
  _that.fetch(function(){}, null);
}

