/******************************************************
 * Copyright eNoe 2011 - All rights reserved *
 * Author: Séverin MARCOMBES                          *
 ******************************************************/
 /**
 * ToolbarManager
 * @constructor
 * @param {WindowManager} windowManager
 * @param {TrackingManager} trackingManager
 * @param {Database} database
 */
function ToolbarManager(windowManager, trackingManager, database, configManager, eventManager)
{
  var _that               = this;

  _that._windowManager    = windowManager;
  _that._trackingManager  = trackingManager;
  _that._database         = database;
  _that._configManager    = configManager;
  _that._eventManager     = eventManager;
//-------------------------------------------------------------- 
//--------------------------------------------------------------  
  _that._activateWindow     = function(w) 
  {
    ////////////////////////////////////////////////////////
    // Cancel acton if no TOS acceptance
    ////////////////////////////////////////////////////////
    if(_that._configManager.getConfig()['acceptedTos'] === false)
    {
      return;
    }
    ////////////////////////////////////////////////////////

    w['toolbarState']['activated']  = true;
    w['callback']({'action':'activate_toolbar'});    

    //////////////////////////////////////
    // EVENT TRACKING
    //////////////////////////////////////
    navigatorAbstractionLayer.getUrlOfTab
    (
      w['id'],
      function(url)
      {
        _that._eventManager.sendRawTrackingActivationEvent
        (
          _that._trackingManager.getTrackingId(), 
          w['domainId'],
          url
        );
      }
    );
    //////////////////////////////////////
  };
//--------------------------------------------------------------  
  _that._desactivateWindow     = function(w) 
  {
    ////////////////////////////////////////////////////////
    // Cancel acton if no TOS acceptance
    ////////////////////////////////////////////////////////
    if(_that._configManager.getConfig()['acceptedTos'] === false)
    {
      return;
    }
    ////////////////////////////////////////////////////////
    
    w['toolbarState']['activated']  = false;
    w['callback']({'action':'desactivate_toolbar'});    
  };
//--------------------------------------------------------------  
  _that.activateDomain     = function(domainId)
  {
    var ws = _that._windowManager.getWindowsByDomainId(domainId);
    
    if((ws)&&(ws[0]))
    {
      for(var i=0; i < ws.length; i++)
      {
        _that._activateWindow(ws[i]);
      }
    }
  }
//--------------------------------------------------------------  
  _that.desactivateDomain     = function(domainId)
  {
    var ws = _that._windowManager.getWindowsByDomainId(domainId);
    
    if((ws)&&(ws[0]))
    {
      for(var i=0; i < ws.length; i++)
      {
        _that._desactivateWindow(ws[i]);
      }
    }
  }
//--------------------------------------------------------------  
  _that.desactivateAll       = function()
  {
    var ws = _that._windowManager.getAllWindows();
    
    if((ws)&&(ws[0]))
    {
      for(var i=0; i < ws.length; i++)
      {
        _that._desactivateWindow(ws[i]);
      }
    }
  }
//--------------------------------------------------------------  
  _that.initWindow         = function(windowId,referrer_result)
  {
    ////////////////////////////////////////////////////////
    // Cancel acton if no TOS acceptance
    ////////////////////////////////////////////////////////
    if(_that._configManager.getConfig()['acceptedTos'] === false)
    {
      return;
    }
    ////////////////////////////////////////////////////////
    
    var w = _that._windowManager.getWindowById(windowId);

    if(w)
    {

      //Init the toolbar object
      w['toolbarState']                 = new Object();
      w['toolbarState']['activated']    = _that._trackingManager.isDomainActive(w['domainId']);
      w['toolbarState']['_displayed']   = true;
      w['toolbarState']['isReferrerBlock']   = referrer_result['block'];   
      w['toolbarState']['referrerMessage']   = referrer_result['message'];    
      console.log(referrer_result['block']+"#"+referrer_result['message']);
      
      _that._database.getDomainInfoById
      (
        w['domainId'],
        
        function(info, args)
        {
          var w = args['w'];
          
          w['toolbarState']['_barText'] 			 = info['title'];
          w['toolbarState']['_promoDisplayed']	= (info['codes'].length>0);
          
          //Display the toolbar on the remote window
          w['callback']({'action':'display_toolbar', 'args':{'toolbarState':w['toolbarState']}});  
        },

        {
          'w':        w
        }
      );  
    } 
  };
//--------------------------------------------------------------  
  _that.isWindowActive         = function(windowId)
  {
    var w = _that._windowManager.getWindowById(windowId);

    if(w)
    {

      if(_that._trackingManager.isDomainActive(w['domainId']))
      {
         w['callback']({'action':'activate_toolbar', 'args':{'toolbarState':w['toolbarState']}}); 

        //////////////////////////////////////
        // EVENT TRACKING
        //////////////////////////////////////
        navigatorAbstractionLayer.getUrlOfTab
        (
          w['id'],
          function(url)
          {
            _that._eventManager.sendRawTrackingActivationEvent
            (
              _that._trackingManager.getTrackingId(), 
              w['domainId'],
              url
            );
          }
        );
        //////////////////////////////////////
      }
      else
      {
        w['callback']({'action':'desactivate_toolbar', 'args':{'toolbarState':w['toolbarState']}}); 
      }
    } 
  };
}

