/******************************************************
 * Copyright eNoe 2011 - All rights reserved *
 * Author: Séverin MARCOMBES                          *
 ******************************************************/

/**
 * TrackingManager
 * @constructor
 * @param {WindowManager} windowManager
 * @param {Database} database
 * @param {ConfigManager} configManager
 * @param {int} updatePeriod
 */
function TrackingManager(windowManager, database, configManager, updatePeriod)
{
  var _that                       = this;

  _that._windowManager            = windowManager;
  _that._domainMap                = new Object();
  _that._database                 = database;
  _that._configManager            = configManager;
  _that._trackingId               = 0;
  _that.updatePeriod              = updatePeriod;
  _that.timer                     = null;
  _that._windowsBeingActivated    = {};
  _that._windowsToRedirect        = {};
//-------------------------------------------------------------- 
//-------------------------------------------------------------- 
  _that._activateWindow     = function(w)
  {   
      var req               = new Object();
      
      req['action']         = 'activate_tracking';
      req['tabId']          = w['id'];
      req['args']           = new Object();
      
      if(_that._domainMap[w['domainId']]['trackingUrls'].length > 0)
      {        
          req['args']['trackingUrls'] = _that._domainMap[w['domainId']]['trackingUrls'];
          w['callback'](req);
          
          ////////
          _that._windowsBeingActivated[w['id']] = w['domainId'];
          ////////
      }
      else
      {
        //Get the tracking URLs if nobody fetched them before
        _that._database.getDomainInfoById
        (
          w['domainId'],
        
          function(info, args)
          {
            var w     = args['w'];
            
            //Now we've got the tracking URL, let's replace the %id% tag in them
            var urls=new Array();
           
            var reg=new RegExp("(%id%)", "g");
            
            for(var i=0; i<info['urls'].length; i++)
            {
              urls.push(info['urls'][i].replace(reg,"" + _that._trackingId));
            }
            
            _that._domainMap[w['domainId']]['trackingUrls'] = urls;
            
            if(_that._domainMap[w['domainId']]['trackingUrls'].length > 0)
            {
              _that._activateWindow(w);
            }
          },
          {
            '_this':           _that,
            'w':               w
          }
        );  
      }    
  }; 
//-------------------------------------------------------------- 
  _that._desactivateWindow     = function(w)
  {
    var req           = 
    {
      'action':       'desactivate_tracking',
      'args':         {}
    };
   
    w['callback'](req);
  }
//-------------------------------------------------------------- 
  _that.initWindow         = function(windowId)
  {
    var w = _that._windowManager.getWindowById(windowId);
    
    //Map the domain
      if(!_that._domainMap[w['domainId']])
      {
        _that._domainMap[w['domainId']] = 
        {
          'activated':      false,
          'trackingUrls':   [],
          'subDomains':     {}
        };
      }
      
      if(!_that._domainMap[w['domainId']]['subDomains'][w['subDomain']])
      {
        _that._domainMap[w['domainId']]['subDomains'][w['subDomain']] =
        {
          'activated': false
        };
      }
      
      
    ///////////////////////////  
    //React to windows that have just been activated
    if((_that._windowsToRedirect[windowId])&&(_that._windowsToRedirect[windowId] != ""))
    {
      _that._domainMap[w['domainId']]['activated'] = true;
      _that._domainMap[w['domainId']]['subDomains'][w['subDomain']]['activated'] = true;
      
      var req           = 
      {
        'action':       'after_tracking',
        'args':         
        {
          'url':        _that._windowsToRedirect[windowId]
        }
      };
   
      w['callback'](req);
      
      _that._windowsToRedirect[windowId] = "";
    } 
    ///////////////////////////  
    else
    {
         
      //If the tracking should be activated, launch a request
      if(_that._domainMap[w['domainId']]['activated'])
      {
        //////////////////
        if((!_that._windowsBeingActivated[w['id']])||(_that._windowsBeingActivated[w['id']] != w['domainId']))
        {
            
        //////////////////
      

          _that._activateWindow(w);
        }
        
        if(!_that._domainMap[w['domainId']]['subDomains'][w['subDomain']]['activated'])
        {
            _that._domainMap[w['domainId']]['subDomains'][w['subDomain']]['activated'] = true;
            
        }
      }
    }
  };
//-------------------------------------------------------------- 
/////////////////////////// 
  _that.saveAfterTrackingUrl     = function(windowId, url)
  {
    _that._windowsToRedirect[windowId] = url;
  };
/////////////////////////// 
//--------------------------------------------------------------   
  _that.activateDomain     = function(domainId)
  {
    if(_that._domainMap[domainId])
    {
      // if(!_that._domainMap[domainId]['activated'])
      // {
          _that._domainMap[domainId]['activated'] = true;
          
          for(var subDomain in _that._domainMap[domainId]['subDomains'])
          {
            //if(!subDomain['activated'])
            //{
              subDomain['activated'] = true;
              
              ws = _that._windowManager.getWindowsBySubDomain(subDomain);
              
              //if((ws)&&(ws[0]))
              for(var i=0; i < ws.length; i++)
              {
              ////////////
                if((!_that._windowsBeingActivated[ws[i]['id']])||(_that._windowsBeingActivated[ws[i]['id']] != ws[i]['domainId']))
                {
                //////////
                  _that._activateWindow(ws[i]);
                //////////
                }
             /////////////
              }
           // }
      //    }
          
          var activatedDomainsJSON		= _that._configManager.getConfig()['activatedDomains'];
          
          var activatedDomains			= new Array();
          
          if((activatedDomainsJSON)&&(activatedDomainsJSON!=""))
          {
          	activatedDomains = JSON.parse(activatedDomainsJSON);
          }
          
          activatedDomains.push(domainId+"");
          _that._configManager.setConfig({'activatedDomains':JSON.stringify(activatedDomains)});
       }
    }
  };
//--------------------------------------------------------------   
  _that.isDomainActive     = function(domainId)
  {
    return ((_that._domainMap[domainId])&&(_that._domainMap[domainId]['activated']));
  };
//--------------------------------------------------------------   
  _that.setTrackingId     = function(trackingId)
  {
    _that._trackingId = trackingId;
  };
//--------------------------------------------------------------
  _that.getTrackingId     = function()
  {
    return _that._trackingId;
  };
//--------------------------------------------------------------   
  _that.reset             = function()
  {
    _that.setTrackingId(0);
    _that._domainMap = new Object();
    
    //////////////////
    _that._windowsBeingActivated = {};
    _that._windowsToRedirect = {};
    //////////////////
   
  };
//--------------------------------------------------------------   
  _that.desactivateAll    = function()
  {
    _that.reset();
    
    _that._configManager.setConfig({'activatedDomains':""});
    
    var ws = _that._windowManager.getAllWindows();
    
    if((ws)&&(ws[0]))
    {
      for(var i=0; i < ws.length; i++)
      {
        _that.initWindow(ws[i]['id']);
        _that._desactivateWindow(ws[i]);
      }
    }
  };
//--------------------------------------------------------------   
  _that.reActivateAll    = function()
  {
    var ws = _that._windowManager.getAllWindows();
    
    if((ws)&&(ws[0]))
    {
      for(var i=0; i < ws.length; i++)
      {
        if(_that.isDomainActive(ws[i]['domainId']))
        {
          _that._activateWindow(ws[i]);
        }
      }
    }
  };  
//--------------------------------------------------------------
      _that.setUpdatePeriod    = function(updatePeriod)
      {
      //DISABLED TEMPORARILY
      /*
        _that.updatePeriod   = updatePeriod;
        
        clearInterval(_that.timer);
       
        _that.timer				= setInterval
        (
          function()
          {
            _that.reActivateAll();
          },
          
          (_that.updatePeriod * 1000)
        );	
      */
      };
//--------------------------------------------------------------
//--------------------------------------------------------------

			//Set a timer to refresh the tracking periodically
      _that.setUpdatePeriod(_that.updatePeriod);
  
}

