/******************************************************
 * Copyright eNoe 2011 - All rights reserved *
 * Author: Séverin MARCOMBES                          *
 ******************************************************/
/**
 * WindowManager
 * @constructor
 */
function WindowManager()
{
  var _that   = this;

  _that._mapByDomain          = new Object();
  _that._mapById              = new Object();
//--------------------------------------------------------------
//--------------------------------------------------------------  
  _that.addWindow             = function(id, domainId, subDomain)
  {
    //Create the new window's data structure
    var windowData = 
    {
      'id':                 id,
      'domainId':           domainId,
      'subDomain':          subDomain,
      'callback':           function(request)
      {
        navigatorAbstractionLayer.sendRequestToTab(this['id'], request);
      }
    };
  
    //Map by domain
    if(!_that._mapByDomain[domainId])
    {
      _that._mapByDomain[domainId] = new Object();
    }
    
    if(!_that._mapByDomain[domainId][subDomain])
    {
      _that._mapByDomain[domainId][subDomain] = new Object();
    }
    
    if(!_that._mapByDomain[domainId][subDomain][id])
    {
      _that._mapByDomain[domainId][subDomain][id] = windowData;
    }
    
    //Map by id
    if(!_that._mapById[id])
    {
      _that._mapById[id] = windowData;
    }
  };
//--------------------------------------------------------------    
  _that.removeWindow          = function(id)
  {
    if(_that._mapById[id])
    {
      //Remove by domain
      if(_that._mapByDomain[_that._mapById[id]['domainId']][_that._mapById[id]['subDomain']][id])
      {
        delete _that._mapByDomain[_that._mapById[id]['domainId']][_that._mapById[id]['subDomain']][id];
      }
    
      //Remove by id
      delete _that._mapById[id];
    }
  };
//--------------------------------------------------------------       
  _that.updateWindowCallback   = function(id, callback)
  {
    if(_that._mapById[id])
    {
      _that._mapById[id]['callback'] = callback;
    }
  };
//--------------------------------------------------------------       
  _that.getWindowById          = function(id)
  {
    var res = null;
    
    if(_that._mapById[id])
    {
      return _that._mapById[id];
    }
    
    return res;
  };
//--------------------------------------------------------------       
  _that.getAllWindows          = function()
  {
    var res = new Array();
    
    for(var w in _that._mapById)
    {
      res.push(_that._mapById[w]);
    }
    
    return res;
  };
//--------------------------------------------------------------       
  _that.getWindowsByDomainId    = function(domainId)
  {
    var res = new Array();
    
    if(_that._mapByDomain[domainId])
    {
      for(var subDomain in _that._mapByDomain[domainId])
      {
        for(var windowData in _that._mapByDomain[domainId][subDomain])
        {
          res.push(_that._mapByDomain[domainId][subDomain][windowData]);
        }
      }
    }
    
    return res;
  };
//--------------------------------------------------------------       
  _that.getWindowsBySubDomain   = function(subDomain)
  {
    var res = new Array();
    
    for(var windowData in _that._mapById)
    {
      if(_that._mapById[windowData]['subDomain'] == subDomain)
      {
        res.push(_that._mapById[windowData]);
      }
    }
    
    return res;
  };
//--------------------------------------------------------------    
}
